(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is TurboPower OnGuard
 *
 * ***** END LICENSE BLOCK ***** *)

{**********************************************************************
 Package pl_OnGuard.pkg
 This unit is part of CodeTyphon Project (http://www.pilotlogic.com/)
***********************************************************************}

{$I onguard.inc}


unit ogfirst;

interface

uses
{$IFDEF WINDOWS}
  Windows,
{$ENDIF}
{$IFDEF UNIX}
  BaseUnix,
//  Libc,//only because  S_IRWXU was missing from baseunix under 0.9.16
  //fix it later
{$ENDIF}
  Forms, SysUtils, Dialogs, LClProc;


const
 MAGIC = 'MAGIC'; {change this code to differentiate applications}


{detect/Activate instance routines}
function  IsFirstInstance: Boolean;
procedure ActivateFirstInstance;

Procedure _Init_ogfirst;
Procedure _Stop_ogfirst;

implementation


{$IFDEF WINDOWS}
var
  FirstInstance : Boolean;
  InstanceMutex : THandle;
{$ENDIF}

{$IFDEF UNIX}
var
  FirstInstance : Boolean;
  server_name : String;
  server_lock : Integer;

{$ENDIF}

{limit instances routines}
function IsFirstInstance : Boolean;
begin
  {$IFDEF WINDOWS}
  Result := FirstInstance;
  {$ELSE}
  {$IFDEF UNIX}
  Result := FirstInstance;
  {$ELSE}
  Result := HPrevInst = 0;
  {$ENDIF}
  {$ENDIF}
end;

{$IFDEF WINDOWS}
procedure ActivateFirstInstance;
var
  ClassBuf,
  WindowBuf : array [0..255] of AnsiChar;
  Wnd,
  TopWnd    : hWnd;
  ThreadID  : DWord;
begin
try
  if IsFirstInstance then begin
    if IsIconic(HWND(Application.MainForm.Handle)) then
      ShowWindow(HWND(Application.MainForm.Handle), SW_RESTORE)// Application.Restore
    else
      Application.BringToFront;
  end else begin
    GetClassName(HWND(Application.MainForm.Handle), ClassBuf, SizeOf(ClassBuf));
    GetWindowText(HWND(Application.MainForm.Handle), WindowBuf, SizeOf(WindowBuf));
    Wnd := FindWindow(ClassBuf, WindowBuf);
    if (Wnd <> 0) then begin
      GetWindowThreadProcessId(Wnd, @ThreadID);
      if (ThreadID = GetCurrentProcessId) then begin
        Wnd := FindWindowEx(0, Wnd, ClassBuf, WindowBuf);
        if (Wnd <> 0) then
          if IsIconic(Wnd) then
            ShowWindow(Wnd, SW_RESTORE)
          else begin
            SetForegroundWindow(Wnd);
            TopWnd := GetLastActivePopup(Wnd);
            if (TopWnd <> 0) and (TopWnd <> Wnd) and
                IsWindowVisible(TopWnd) and IsWindowEnabled(TopWnd) then
              BringWindowToTop(TopWnd)
            else
              BringWindowToTop(Wnd);
          end;
      end;
    end;
  end;
except on E:Exception do
 DebugLn('ActivateFirstInstance exception : ' + E.Message + '.Move IsFirstInstance after CreateForm for MainForm');
end;
end;
{$ELSE}
procedure ActivateFirstInstance;
begin
 //[to do] Find and Activate the first instance of the application

 //look at the owner of the socket
 //look at the running processes
end;
{$ENDIF}


//=========================================================================

{$IFDEF WINDOWS}
function GetMutexName : string;
var
  WindowBuf : array [0..512] of AnsiChar;
begin
try
  {GetWindowText(HWND(Application.MainForm.Handle), WindowBuf, SizeOf(WindowBuf));}
  Result := 'PREVINST:' + ExtractFileName(ParamStr(0)) + MAGIC;
except on E:Exception do
 DebugLn('GetMutexName exception : '  + E.Message);
end;
end;


Procedure _Init_ogfirst;
begin
  InstanceMutex := CreateMutex(nil, True, PAnsiChar(GetMutexName));
  if (InstanceMutex <> 0) and (GetLastError = 0) then
    FirstInstance := True
  else
    FirstInstance := False;
end;

Procedure _Stop_ogfirst;
begin
  if (InstanceMutex <> 0) then
    CloseHandle(InstanceMutex);
end;

{$ENDIF}

{$IFDEF UNIX}
Procedure _Init_ogfirst;
begin
 server_name := ExtractFilePath(ParamStr(0)) +  ExtractFileName(ParamStr(0)) + '.lck';  
  {$IF DEFINED(SOLARIS) or (DEFINED(DARWIN) or DEFINED(iPHONESIM))}  // ct9999
  // O_NOFOLLOW = Don't follow symlinks and
  // S_IRWXU = S_IRUSR or S_IWUSR or S_IXUSR
  // NOT Exists on Solaris
  server_lock := fpopen(PChar(server_name), O_RDWR or O_CREAT or O_TRUNC {or O_NOFOLLOW} or O_EXCL, S_IRUSR or S_IWUSR or S_IXUSR);
  {$ELSE}
  server_lock := fpopen(PChar(server_name), O_RDWR or O_CREAT or O_TRUNC or O_NOFOLLOW or O_EXCL, S_IRWXU);
  {$ENDIF}
 if (server_lock = -1) then
 begin
  FirstInstance := False;
  DebugLn('Failed to create lock file. (' + IntToHex(errno,4) + ')' + #10 + server_name);
 end
 else
 begin
  FirstInstance := True;
 end;
 end;


Procedure _Stop_ogfirst;
begin
 if (server_lock > -1) then
 begin
  FileClose(server_lock);
//  unlink(PChar(server_name)); // ct9999
 end;
end;

{$ENDIF}

end.
